//
//  CCItemsViewController.m
//  iCloudDemoMOBC2013
//
//  Created by Gianluca Tranchedone on 20/10/2013.
//  Copyright (c) 2013 Cocoa Beans GT Limited. All rights reserved.
//

#import "CCItemsViewController.h"

@implementation CCItemsViewController

#pragma mark - Private APIs

- (IBAction)addItemAction:(UIBarButtonItem *)sender
{
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Create Item"
                                                        message:@"Type the item's title"
                                                       delegate:self
                                              cancelButtonTitle:@"Cancel"
                                              otherButtonTitles:@"Create", nil];
    alertView.alertViewStyle = UIAlertViewStylePlainTextInput;
    
    [alertView show];
}

#pragma mark - UIAlertViewDelegate

- (BOOL)alertViewShouldEnableFirstOtherButton:(UIAlertView *)alertView
{
    return [alertView textFieldAtIndex:0].text.length > 0;
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex == alertView.firstOtherButtonIndex) {
        NSManagedObjectContext *context = [CCCloudManager sharedManager].managedObjectContext;
        CCItem *newItem = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass([CCItem class]) inManagedObjectContext:context];
        newItem.title = [alertView textFieldAtIndex:0].text;
        newItem.creationDate = [NSDate date];
        newItem.list = self.list;
        
        [[CCCloudManager sharedManager] saveContext];
    }
}

#pragma mark - UITableViewDataSource

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    CCItem *item = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    cell.textLabel.text = item.title;
    
    return cell;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    CCItem *item = [self.fetchedResultsController objectAtIndexPath:indexPath];
    item.completionDate = [NSDate date];
    
    [[CCCloudManager sharedManager] saveContext];
}

#pragma mark - NSFetchedResultsControllerDelegate

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    [super controllerDidChangeContent:controller];
    
    if ([self.list isDeleted]) {
        [self.navigationController popViewControllerAnimated:YES];
        return;
    }
}

#pragma mark - Setters and Getters

- (NSFetchedResultsController *)fetchedResultsController
{
    if (![super fetchedResultsController]) {
        NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:NSStringFromClass([CCItem class])];
        fetchRequest.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"creationDate" ascending:YES]];
        fetchRequest.predicate = [NSPredicate predicateWithFormat:@"completionDate = nil"];
        
        if (self.list) {
            fetchRequest.predicate = [NSCompoundPredicate andPredicateWithSubpredicates:@[fetchRequest.predicate,
                                                                                          [NSPredicate predicateWithFormat:@"list = %@", self.list]]];
        }
        
        NSFetchedResultsController *fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:[CCCloudManager sharedManager].managedObjectContext sectionNameKeyPath:nil cacheName:nil];
        fetchedResultsController.delegate = self;
        
        self.fetchedResultsController = fetchedResultsController;
    }
    
    return [super fetchedResultsController];
}

@end
