//
//  CCListsViewController.m
//  iCloudDemoMOBC2013
//
//  Created by Gianluca Tranchedone on 20/10/2013.
//  Copyright (c) 2013 Cocoa Beans GT Limited. All rights reserved.
//

#import "CCListsViewController.h"
#import "CCItemsViewController.h"

@implementation CCListsViewController

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    CCItemsViewController *viewController = segue.destinationViewController;
    NSIndexPath *indexPath = [self.tableView indexPathForCell:sender];
    CCList *list = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    viewController.list = list;
}

#pragma mark - Private APIs

- (IBAction)addListAction:(UIBarButtonItem *)sender
{
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Create List"
                                                        message:@"Type the list's name"
                                                       delegate:self
                                              cancelButtonTitle:@"Cancel"
                                              otherButtonTitles:@"Create", nil];
    alertView.alertViewStyle = UIAlertViewStylePlainTextInput;
    
    [alertView show];
}

#pragma mark - UIAlertViewDelegate

- (BOOL)alertViewShouldEnableFirstOtherButton:(UIAlertView *)alertView
{
    return [alertView textFieldAtIndex:0].text.length > 0;
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex == alertView.firstOtherButtonIndex) {
        NSManagedObjectContext *context = [CCCloudManager sharedManager].managedObjectContext;
        CCList *newList = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass([CCList class]) inManagedObjectContext:context];
        newList.title = [alertView textFieldAtIndex:0].text;
        
        [[CCCloudManager sharedManager] saveContext];
    }
}

#pragma mark - UITableViewDataSource

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    CCList *list = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    cell.textLabel.text = list.title;
    
    return cell;
}

#pragma mark - Setters and Getters

- (NSFetchedResultsController *)fetchedResultsController
{
    if (![super fetchedResultsController]) {
        NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:NSStringFromClass([CCList class])];
        fetchRequest.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"title" ascending:YES]];
        
        NSFetchedResultsController *fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:[CCCloudManager sharedManager].managedObjectContext sectionNameKeyPath:nil cacheName:nil];
        fetchedResultsController.delegate = self;
        
        self.fetchedResultsController = fetchedResultsController;
    }
    
    return [super fetchedResultsController];
}

@end
